" Menu Translations:	Traditional Chinese (for UNIX/Big5 Windows)
" Last Change:		2000 Nov 11

" $LANG  on Windows 95/98/NT is Chinese(Taiwan)_Taiwan.950
" $LANG  on Windows 2000/ME is Chinese_Taiwan.950
" Source the other one from here.
source <sfile>:p:h/menu_chinese_taiwan.950.vim
